% The program needs the speaker output calculated in wfs_presimulation.m
% and the same impulse responses of the loudspeaker like used in the
% presimulation and in the calculation of the driving functions. For each
% point of the simulation area the pressure coming from each loudspeaker is
% added up. The respective pressure is calculated by delaying the speaker
% output in the respective angle according to the distance to the loudspeaker.
% Linear interpolation can be used for points between two angels or two samples. 
% Finally the listening area is drawn and a *.tif file with the normalized
% pressure field is written for a defined snapshot time.
% Alternative a *.wav file can be written for the time distribution of the 
% pressure in a defined listener position.
% All definitions are contained in simulation_definitions.m

clear all;
close all;
clc;

tic  % Start measurement of time.

load speaker_output;

simulation_definitions;  

length_speaker_output = size(speaker_output,3);
speaker_output_used = zeros(1,no_speakers);
speaker_output_used_smaller_angle_earlier_sample = zeros(1,no_speakers);
speaker_output_used_smaller_angle_later_sample = zeros(1,no_speakers);
speaker_output_used_larger_angle_earlier_sample = zeros(1,no_speakers);
speaker_output_used_larger_angle_later_sample = zeros(1,no_speakers);

output_simulation_snapshot_mask = zeros(1+size_area/snapshot_resolution, 1+size_area/snapshot_resolution);

if simulation_type==1  % Creates a *.tif picture over the simulation area for a specific snapshot time.
    output_simulation_snapshot = zeros(1+size_area/snapshot_resolution, 1+size_area/snapshot_resolution);
    %output_simulation_snapshot_mask = sparse(1+size_area/snapshot_resolution, 1+size_area/snapshot_resolution);  % Conserves memory.
    j=0;
    for x = area_start_x : snapshot_resolution : area_end_x,      % Scanns each point of the simulation area.
        j=j+1;                                                    %    
        i=0;                                                      %
        for z = area_start_z : snapshot_resolution : area_end_z,  %
            i=i+1;                                                %  
            distance_to_speaker = sqrt((z).^2 + (x_speaker_position - x).^2);
            if min(distance_to_speaker) < distance_threshold
               output_simulation_snapshot_mask(i,j) = 1;          % Makes the pixels close to the speaker white.
               speaker_output_used=zeros(1,no_speakers);
            else
                angle_to_speaker = (180 / pi) * asin(abs(x_speaker_position - x) ./ distance_to_speaker);
            
                if interpolation==0,  % No interpolation for positions between two angels and two time samples.
                    angle_used = round(angle_to_speaker / measurement_step_degrees) * measurement_step_degrees;
                    angle_used_index = angle_used ./ measurement_step_degrees;  % Nearest neighbor is used.
                    no_sample_used = 1 + snapshot_time - round(distance_to_speaker ./ speed_of_sound); 
                    for k = 1 : no_speakers,
                        if (no_sample_used(k) < 1) || (no_sample_used(k) > length_speaker_output)
                            speaker_output_used(k)=0;
                        else
                            speaker_output_used(k) = speaker_output(k,angle_used_index(k)+1,no_sample_used(k))/distance_to_speaker(k);
                        end
                    end
                else  % Linear interpolation is used for positions between two angels and two time samples. 
                    angle_used = angle_to_speaker;
                    angle_used_index = angle_used ./ measurement_step_degrees;
                    no_sample_used = 1 + snapshot_time - distance_to_speaker ./ speed_of_sound;
        
                    weight_smaller_angle = 1 - (angle_used_index - floor(angle_used_index));  
                    weight_larger_angle =  1 - (ceil(angle_used_index) - angle_used_index);
                    weight_earlier_sample = 1 - (no_sample_used - floor(no_sample_used));
                    weight_later_sample = 1 - (ceil(no_sample_used) - no_sample_used);
        
                    for k = 1 : no_speakers,
                        if (no_sample_used(k) < 1) || (no_sample_used(k) > length_speaker_output)
                            speaker_output_used(k)=0;
                        else
                            speaker_output_used_smaller_angle_earlier_sample(k) = speaker_output(k,(floor(angle_used_index(k)))+1,floor(no_sample_used(k))) / distance_to_speaker(k);
                            speaker_output_used_smaller_angle_later_sample(k) = speaker_output(k,(floor(angle_used_index(k)))+1,ceil(no_sample_used(k))) / distance_to_speaker(k);
                            speaker_output_used_larger_angle_earlier_sample(k) = speaker_output(k,(ceil(angle_used_index(k)))+1,floor(no_sample_used(k))) / distance_to_speaker(k);
                            speaker_output_used_larger_angle_later_sample(k) = speaker_output(k,(ceil(angle_used_index(k)))+1,ceil(no_sample_used(k))) / distance_to_speaker(k);
                            speaker_output_used(k) = weight_smaller_angle(k) .* weight_earlier_sample(k) .* speaker_output_used_smaller_angle_earlier_sample(k);
                            speaker_output_used(k) = weight_larger_angle(k) .* weight_earlier_sample(k) .* speaker_output_used_larger_angle_earlier_sample(k) + speaker_output_used(k);
                            speaker_output_used(k) = weight_smaller_angle(k) .* weight_later_sample(k) .* speaker_output_used_smaller_angle_later_sample(k) + speaker_output_used(k);
                            speaker_output_used(k) = weight_larger_angle(k) .* weight_later_sample(k) .* speaker_output_used_larger_angle_later_sample(k) + speaker_output_used(k);
                        end
                    end
                end
            end
            output_simulation_snapshot(i,j) = sum(speaker_output_used);  % Superposition of all loudspeaker influences in one simulation point.
        end
    end
    
    output_simulation_snapshot = 0.5*output_simulation_snapshot/max(max(abs(output_simulation_snapshot)));  % Reduces the range from -0.5 until 0.5.
    %output_simulation_snapshot = 0.5*output_simulation_snapshot/0.0025;  % Only for animation.
    output_simulation_snapshot = output_simulation_snapshot * 1;  % Multiplication with other factors than one will increase the contrast of the picture. 
    output_simulation_snapshot = output_simulation_snapshot + 0.5;  % Shifts the range to zero until one. 0.5 equals no pressure.
    
    % Draws the listening area into the picture with white lines.
    top_left_pixel_x = round((listarea_top_left_x+(size_area/2))/snapshot_resolution); 
    top_left_pixel_z = round(listarea_top_left_z/snapshot_resolution);
    bottom_right_pixel_x = round((listarea_bottom_right_x+size_area/2)/snapshot_resolution);
    bottom_right_pixel_z = round(listarea_bottom_right_z/snapshot_resolution);
    output_simulation_snapshot_mask(top_left_pixel_z:bottom_right_pixel_z,top_left_pixel_x)=1;
    output_simulation_snapshot_mask(top_left_pixel_z:bottom_right_pixel_z,bottom_right_pixel_x)=1;
    output_simulation_snapshot_mask(top_left_pixel_z,top_left_pixel_x:bottom_right_pixel_x)=1;
    output_simulation_snapshot_mask(bottom_right_pixel_z,top_left_pixel_x:bottom_right_pixel_x)=1;

    output_simulation_snapshot_mask(round(sampling_point_z / snapshot_resolution),round((sampling_point_x + (size_area/2)) / snapshot_resolution))=1;
    output_simulation_snapshot_mask(round(sampling_point_z / snapshot_resolution) + 1,round((sampling_point_x + (size_area/2)) / snapshot_resolution) - 1)=1;
    output_simulation_snapshot_mask(round(sampling_point_z / snapshot_resolution) - 1,round((sampling_point_x + (size_area/2)) / snapshot_resolution) - 1)=1;
    output_simulation_snapshot_mask(round(sampling_point_z / snapshot_resolution) + 1,round((sampling_point_x + (size_area/2)) / snapshot_resolution) + 1)=1;
    output_simulation_snapshot_mask(round(sampling_point_z / snapshot_resolution) - 1,round((sampling_point_x + (size_area/2)) / snapshot_resolution) + 1)=1;
    
    for i = 1:size(output_simulation_snapshot,2),
        for j = 1:size(output_simulation_snapshot,1),
            if output_simulation_snapshot_mask(i,j) > 0
                output_simulation_snapshot(i,j) = output_simulation_snapshot_mask(i,j);
            end
        end
    end
    
    filename = strcat('simulation_','sizearea',num2str(size_area),'_',simulation_description,'_snapshottime',num2str(snapshot_time));
    imwrite(output_simulation_snapshot,strcat('input_output/',filename,'.tif'),'tiff','Compression','packbits','Description',filename);

elseif simulation_type == 2  % Creates a *.wav file in a specific listener position. 
    output_simulation_wave = zeros(1,1 + sampling_time_end - sampling_time_start);

    x = sampling_point_x;
    z = sampling_point_z;
    
    distance_to_speaker = sqrt((z).^2 + (x_speaker_position - x).^2);
    angle_to_speaker = (180 / pi) * asin(abs(x_speaker_position - x) ./ distance_to_speaker);

    angle_used = angle_to_speaker;
    angle_used_index = angle_used ./ measurement_step_degrees;
    weight_smaller_angle = 1 - (angle_used_index - floor(angle_used_index));   % Linear interpolation is used like above.
    weight_larger_angle =  1 - (ceil(angle_used_index) - angle_used_index);    %

	for snapshot_time = sampling_time_start : sampling_time_end,
        no_sample_used = 1 + snapshot_time - distance_to_speaker ./ speed_of_sound;
	
        weight_earlier_sample = 1 - (no_sample_used - floor(no_sample_used));  %
        weight_later_sample = 1 - (ceil(no_sample_used) - no_sample_used);     %
	
        for k = 1 : no_speakers,
            if (no_sample_used(k) < 1) || (no_sample_used(k) > length_speaker_output)
                speaker_output_used(k)=0;
            else
                speaker_output_used_smaller_angle_earlier_sample(k) = speaker_output(k,(floor(angle_used_index(k)))+1,floor(no_sample_used(k))) / distance_to_speaker(k);
                speaker_output_used_smaller_angle_later_sample(k) = speaker_output(k,(floor(angle_used_index(k)))+1,ceil(no_sample_used(k))) / distance_to_speaker(k);
                speaker_output_used_larger_angle_earlier_sample(k) = speaker_output(k,(ceil(angle_used_index(k)))+1,floor(no_sample_used(k))) / distance_to_speaker(k);
                speaker_output_used_larger_angle_later_sample(k) = speaker_output(k,(ceil(angle_used_index(k)))+1,ceil(no_sample_used(k))) / distance_to_speaker(k);
                speaker_output_used(k) = weight_smaller_angle(k) .* weight_earlier_sample(k) .* speaker_output_used_smaller_angle_earlier_sample(k);
                speaker_output_used(k) = weight_larger_angle(k) .* weight_earlier_sample(k) .* speaker_output_used_larger_angle_earlier_sample(k) + speaker_output_used(k);
                speaker_output_used(k) = weight_smaller_angle(k) .* weight_later_sample(k) .* speaker_output_used_smaller_angle_later_sample(k) + speaker_output_used(k);
                speaker_output_used(k) = weight_larger_angle(k) .* weight_later_sample(k) .* speaker_output_used_larger_angle_later_sample(k) + speaker_output_used(k);
            end
        end
        output_simulation_wave(snapshot_time) = sum(speaker_output_used);
	end

    filename = strcat('simulation_','sizearea',num2str(size_area),'_',simulation_description,'_x',num2str(x),'_z',num2str(z));
    wavwrite(output_simulation_wave,sample_frequency,16,strcat('input_output/',filename,'.wav'));
end

toc    % End measurement of time.
